package dev.flyfish.framework.config;

import dev.flyfish.framework.config.encrypt.AESEncryptablePropertyResolver;
import com.ulisesbocchio.jasyptspringboot.EncryptablePropertyFilter;
import com.ulisesbocchio.jasyptspringboot.EncryptablePropertyResolver;
import com.ulisesbocchio.jasyptspringboot.annotation.EnableEncryptableProperties;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.autoconfigure.data.mongo.MongoDataAutoConfiguration;
import org.springframework.boot.autoconfigure.data.r2dbc.R2dbcDataAutoConfiguration;
import org.springframework.context.annotation.Bean;

/**
 * 解密配置
 * 沿用datalink优良的安全性
 *
 * @author wangyu
 */
@AutoConfigureBefore({R2dbcDataAutoConfiguration.class, MongoDataAutoConfiguration.class})
@ConditionalOnProperty("flyfish.jasypt.key")
@EnableEncryptableProperties
public class DecryptConfig {

    private static final String PASSWORD_KEY = "password";

    /**
     * 加密属性解析器
     *
     * @return 结果
     */
    @Bean
    public EncryptablePropertyResolver encryptablePropertyResolver(@Value("${flyfish.jasypt.key}") String password) {
        return new AESEncryptablePropertyResolver(password);
    }

    @Bean(name = "encryptablePropertyFilter")
    public EncryptablePropertyFilter encryptablePropertyFilter() {
        return (source, name) -> StringUtils.endsWithIgnoreCase(name, PASSWORD_KEY);
    }
}
