package dev.flyfish.framework.generation.code;

import dev.flyfish.framework.annotations.CodeRule;
import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.generation.CodeRuleStrategy;
import dev.flyfish.framework.generation.code.strategy.TransactionCodeRuleStrategy;
import dev.flyfish.framework.generation.code.strategy.UUIDCodeRuleStrategy;
import dev.flyfish.framework.utils.ReflectionUtils;
import reactor.core.publisher.Mono;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 代码规则工具类
 *
 * @author wangyu
 */
public interface CodeRules {

    // 通用uuid生成
    Class<? extends CodeRuleStrategy> UUID = UUIDCodeRuleStrategy.class;

    // 流水号生成
    Class<? extends CodeRuleStrategy> TRANSACTION = TransactionCodeRuleStrategy.class;

    // 静态策略map
    Map<Class<? extends CodeRuleStrategy>, CodeRuleStrategy> strategies = new ConcurrentHashMap<>();

    /**
     * 获取code
     *
     * @param instance 实体
     * @return 结果
     */
    static String get(Domain instance) {
        CodeRule rule = instance.getClass().getAnnotation(CodeRule.class);
        if (null != rule) {
            CodeRuleStrategy strategy = strategies.computeIfAbsent(rule.value(), ReflectionUtils::instantiate);
            return strategy.generate(instance);
        }
        return null;
    }

    /**
     * 异步获取code
     *
     * @param instance 实体
     * @return 结果
     */
    static Mono<String> retrieve(Domain instance) {
        CodeRule rule = instance.getClass().getAnnotation(CodeRule.class);
        if (null != rule && !rule.value().equals(CodeRuleStrategy.class)) {
            CodeRuleStrategy strategy = strategies.computeIfAbsent(rule.value(), ReflectionUtils::instantiate);
            return strategy.retrieve(instance);
        }
        return Mono.empty();
    }
}
