package dev.flyfish.framework.repository;

import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.query.Queries;
import dev.flyfish.framework.query.Query;
import org.springframework.data.repository.core.EntityInformation;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

/**
 * 高层封装的模型操作
 *
 * @author wangyu
 */
public interface ReactiveEntityOperations {

    /**
     * 查询，通过查询条件和类进行精确查询
     *
     * @param query             查询
     * @param entityInformation 实体信息
     * @param <T>               实体类的实例化结果
     * @param <E>               实体信息具体实现类
     * @return 查询结果
     */
    <T extends Domain, E extends EntityInformation<T, String>> Mono<T> find(Query query, E entityInformation);

    /**
     * 查询全部符合条件的实体
     *
     * @param query             查询
     * @param entityInformation 实体信息
     * @param <T>               实体类的实例化结果
     * @param <E>               实体信息具体实现类
     * @return 结果
     */
    <T extends Domain, E extends EntityInformation<T, String>> Flux<T> findAll(Query query, E entityInformation);

    /**
     * 保存实体
     *
     * @param bean 实体
     * @param <T>  实体类的实例化结果
     * @return 结果
     */
    <T extends Domain> Mono<T> save(T bean);

    /**
     * 转换为原生查询
     *
     * @param query             查询
     * @param entityInformation 实体信息
     * @param <R>               泛型，原生查询类型
     * @param <T>               泛型，实体类型
     * @return 结果
     */
    default <R, T extends Domain> R toNative(Query query, EntityInformation<T, String> entityInformation) {
        return Queries.convert(query, entityInformation);
    }
}
