package dev.flyfish.framework.domain.base;

import com.fasterxml.jackson.annotation.JsonIgnore;
import dev.flyfish.framework.annotations.Generation;
import dev.flyfish.framework.annotations.Property;
import dev.flyfish.framework.validation.groups.CodeValid;
import dev.flyfish.framework.validation.groups.NameValid;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Size;
import lombok.Getter;
import lombok.Setter;
import org.springframework.data.annotation.Id;
import org.springframework.data.annotation.Transient;
import org.springframework.data.mongodb.core.index.Indexed;

import java.io.Serial;
import java.io.Serializable;

@Getter
@Setter
public abstract class Domain implements Po, Named, Serializable {

    @Serial
    private static final long serialVersionUID = -8288256526019424379L;

    /**
     * 主键
     */
    @Id
    @Property(readonly = true)
    @Generation(Generation.Strategy.UUID)
    protected String id;

    /**
     * 编号
     */
    @Indexed(unique = true)
    @Property(title = "编码", inherited = true)
    @NotBlank(message = "编码不可为空", groups = CodeValid.class)
    @Generation(Generation.Strategy.CODE)
    @Size(max = 32)
    protected String code;

    /**
     * 名称
     */
    @Property(title = "名称", inherited = true)
    @NotBlank(message = "名称不可为空", groups = NameValid.class)
    @Generation(Generation.Strategy.NAME)
    @Size(max = 100)
    protected String name;

    /**
     * 上下文冗余，利用内存缓存上下文
     */
    @Transient
    @JsonIgnore
    @Property(readonly = true)
    private IUser currentUser;

    @Override
    public int hashCode() {
        if (id != null) {
            return id.hashCode();
        }
        return super.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (obj instanceof Domain comparing) {
            if (this.getId() == null || comparing.getId() == null) {
                return false;
            }
            return this.getId().equals(comparing.getId());
        }
        return false;
    }
}
